//----------------------------------------------------------------------------
// NAME: 3D Cross Section
// VERSION: 0.5
// DESCRIPTION: Lets the user select a square area of terrain and view it in a separate 3D window.
// DEVELOPER: Erik Newman, Patrick Murris
// WEBSITE: 
//----------------------------------------------------------------------------
// Original Terrain Extractor / .bil exporter by Erik Newman - aka withak
// 0.1	11 Feb 2007		Initial attempt
// 0.2	12 Feb 2007		Add rotated selection area and xml output
// 0.3	14 Feb 2007		Log scale for size slider
// 0.4	14 Feb 2007		Fixed spherical geometry bug at IDL, added configurable filenames
// 0.5  23 Feb 2007		Added image export from WW cache
//----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Diagnostics;
using System.Globalization;
using System.Collections;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using WorldWind;
using WorldWind.Renderable;

namespace TerrainViewer
{
	public class TerrainExtractorForm : Form
	{
		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.Button btnCancel;

		private TerrainExtractorPlugin plugin;
		private Label label1;
		private Label label2;
		private Label label3;
		private NumericUpDown numCenterLat;
		private NumericUpDown numCenterLon;
		private Label label4;
		private TrackBar tbRotation;
		private NumericUpDown numRotation;
		private NumericUpDown numSize;
		private TrackBar tbSize;
		private Label label5;
		private Button btnReset;
		private Button btnRecenter;
		private Label label6;
		private Label label7;
		private TextBox tbFilename;
		private TextBox tbDescription;
		private Label label9;
		private Label label10;
		private Label label11;
		private Label label12;
		private ProgressBar pbarExportProgress;
		private ComboBox comboDEMSize;
		private CheckBox cbTexture;
		private ComboBox comboTextureSize;
		private ComboBox comboLayer;
		private Label label8;
		private GroupBox gbTexture;
		private Button btnRefreshList;


		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.IContainer components = null;

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		/// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
		protected override void Dispose(bool disposing)
		{
			if (disposing && (components != null))
			{
				components.Dispose();
			}
			base.Dispose(disposing);
		}

		#region Windows Form Designer generated code

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnOK = new System.Windows.Forms.Button();
			this.btnCancel = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.numCenterLat = new System.Windows.Forms.NumericUpDown();
			this.numCenterLon = new System.Windows.Forms.NumericUpDown();
			this.label4 = new System.Windows.Forms.Label();
			this.tbRotation = new System.Windows.Forms.TrackBar();
			this.numRotation = new System.Windows.Forms.NumericUpDown();
			this.numSize = new System.Windows.Forms.NumericUpDown();
			this.tbSize = new System.Windows.Forms.TrackBar();
			this.label5 = new System.Windows.Forms.Label();
			this.btnReset = new System.Windows.Forms.Button();
			this.btnRecenter = new System.Windows.Forms.Button();
			this.label6 = new System.Windows.Forms.Label();
			this.label7 = new System.Windows.Forms.Label();
			this.tbFilename = new System.Windows.Forms.TextBox();
			this.tbDescription = new System.Windows.Forms.TextBox();
			this.label9 = new System.Windows.Forms.Label();
			this.label10 = new System.Windows.Forms.Label();
			this.label11 = new System.Windows.Forms.Label();
			this.label12 = new System.Windows.Forms.Label();
			this.pbarExportProgress = new System.Windows.Forms.ProgressBar();
			this.comboDEMSize = new System.Windows.Forms.ComboBox();
			this.cbTexture = new System.Windows.Forms.CheckBox();
			this.comboTextureSize = new System.Windows.Forms.ComboBox();
			this.comboLayer = new System.Windows.Forms.ComboBox();
			this.label8 = new System.Windows.Forms.Label();
			this.gbTexture = new System.Windows.Forms.GroupBox();
			this.btnRefreshList = new System.Windows.Forms.Button();
			((System.ComponentModel.ISupportInitialize)(this.numCenterLat)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numCenterLon)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.tbRotation)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numRotation)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numSize)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.tbSize)).BeginInit();
			this.gbTexture.SuspendLayout();
			this.SuspendLayout();
			// 
			// btnOK
			// 
			this.btnOK.Location = new System.Drawing.Point(99, 275);
			this.btnOK.Name = "btnOK";
			this.btnOK.Size = new System.Drawing.Size(97, 23);
			this.btnOK.TabIndex = 3;
			this.btnOK.Text = "View selection";
			this.btnOK.UseVisualStyleBackColor = true;
			this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
			// 
			// btnCancel
			// 
			this.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.btnCancel.Location = new System.Drawing.Point(202, 275);
			this.btnCancel.Name = "btnCancel";
			this.btnCancel.Size = new System.Drawing.Size(75, 23);
			this.btnCancel.TabIndex = 4;
			this.btnCancel.Text = "Cancel";
			this.btnCancel.UseVisualStyleBackColor = true;
			this.btnCancel.Click += new System.EventHandler(this.btnCancel_Click);
			// 
			// label1
			// 
			this.label1.AutoSize = true;
			this.label1.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.label1.Location = new System.Drawing.Point(16, 9);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(48, 13);
			this.label1.TabIndex = 5;
			this.label1.Text = "Center:";
			// 
			// label2
			// 
			this.label2.AutoSize = true;
			this.label2.Location = new System.Drawing.Point(16, 30);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(25, 13);
			this.label2.TabIndex = 6;
			this.label2.Text = "Lat:";
			// 
			// label3
			// 
			this.label3.AutoSize = true;
			this.label3.Location = new System.Drawing.Point(13, 56);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(28, 13);
			this.label3.TabIndex = 7;
			this.label3.Text = "Lon:";
			// 
			// numCenterLat
			// 
			this.numCenterLat.DecimalPlaces = 5;
			this.numCenterLat.Increment = new decimal(new int[] {
            1,
            0,
            0,
            65536});
			this.numCenterLat.Location = new System.Drawing.Point(46, 28);
			this.numCenterLat.Maximum = new decimal(new int[] {
            90,
            0,
            0,
            0});
			this.numCenterLat.Minimum = new decimal(new int[] {
            90,
            0,
            0,
            -2147483648});
			this.numCenterLat.Name = "numCenterLat";
			this.numCenterLat.Size = new System.Drawing.Size(82, 20);
			this.numCenterLat.TabIndex = 8;
			this.numCenterLat.ValueChanged += new System.EventHandler(this.numCenterLat_ValueChanged);
			// 
			// numCenterLon
			// 
			this.numCenterLon.DecimalPlaces = 5;
			this.numCenterLon.Increment = new decimal(new int[] {
            1,
            0,
            0,
            65536});
			this.numCenterLon.Location = new System.Drawing.Point(46, 54);
			this.numCenterLon.Maximum = new decimal(new int[] {
            180,
            0,
            0,
            0});
			this.numCenterLon.Minimum = new decimal(new int[] {
            180,
            0,
            0,
            -2147483648});
			this.numCenterLon.Name = "numCenterLon";
			this.numCenterLon.Size = new System.Drawing.Size(82, 20);
			this.numCenterLon.TabIndex = 9;
			this.numCenterLon.ValueChanged += new System.EventHandler(this.numCenterLon_ValueChanged);
			// 
			// label4
			// 
			this.label4.AutoSize = true;
			this.label4.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.label4.Location = new System.Drawing.Point(146, 73);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(92, 13);
			this.label4.TabIndex = 10;
			this.label4.Text = "Rotation (deg):";
			// 
			// tbRotation
			// 
			this.tbRotation.Location = new System.Drawing.Point(149, 89);
			this.tbRotation.Maximum = 45;
			this.tbRotation.Minimum = -45;
			this.tbRotation.Name = "tbRotation";
			this.tbRotation.Size = new System.Drawing.Size(160, 37);
			this.tbRotation.SmallChange = 5;
			this.tbRotation.TabIndex = 11;
			this.tbRotation.TickFrequency = 15;
			this.tbRotation.ValueChanged += new System.EventHandler(this.tbRotation_ValueChanged);
			// 
			// numRotation
			// 
			this.numRotation.Increment = new decimal(new int[] {
            5,
            0,
            0,
            0});
			this.numRotation.Location = new System.Drawing.Point(315, 94);
			this.numRotation.Maximum = new decimal(new int[] {
            45,
            0,
            0,
            0});
			this.numRotation.Minimum = new decimal(new int[] {
            45,
            0,
            0,
            -2147483648});
			this.numRotation.Name = "numRotation";
			this.numRotation.Size = new System.Drawing.Size(69, 20);
			this.numRotation.TabIndex = 12;
			this.numRotation.ValueChanged += new System.EventHandler(this.numRotation_ValueChanged);
			// 
			// numSize
			// 
			this.numSize.DecimalPlaces = 4;
			this.numSize.Increment = new decimal(new int[] {
            1,
            0,
            0,
            65536});
			this.numSize.Location = new System.Drawing.Point(315, 30);
			this.numSize.Maximum = new decimal(new int[] {
            60,
            0,
            0,
            0});
			this.numSize.Minimum = new decimal(new int[] {
            1,
            0,
            0,
            131072});
			this.numSize.Name = "numSize";
			this.numSize.Size = new System.Drawing.Size(69, 20);
			this.numSize.TabIndex = 15;
			this.numSize.Value = new decimal(new int[] {
            10,
            0,
            0,
            0});
			this.numSize.ValueChanged += new System.EventHandler(this.numSize_ValueChanged);
			// 
			// tbSize
			// 
			this.tbSize.LargeChange = 500;
			this.tbSize.Location = new System.Drawing.Point(149, 25);
			this.tbSize.Maximum = 1778;
			this.tbSize.Minimum = -2000;
			this.tbSize.Name = "tbSize";
			this.tbSize.Size = new System.Drawing.Size(160, 37);
			this.tbSize.SmallChange = 100;
			this.tbSize.TabIndex = 14;
			this.tbSize.TickFrequency = 500;
			this.tbSize.Value = 1000;
			this.tbSize.ValueChanged += new System.EventHandler(this.tbSize_ValueChanged);
			// 
			// label5
			// 
			this.label5.AutoSize = true;
			this.label5.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.label5.Location = new System.Drawing.Point(146, 9);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(68, 13);
			this.label5.TabIndex = 13;
			this.label5.Text = "Size (deg):";
			// 
			// btnReset
			// 
			this.btnReset.Location = new System.Drawing.Point(198, 142);
			this.btnReset.Name = "btnReset";
			this.btnReset.Size = new System.Drawing.Size(94, 39);
			this.btnReset.TabIndex = 16;
			this.btnReset.Text = "Reset area to camera view";
			this.btnReset.UseVisualStyleBackColor = true;
			this.btnReset.Click += new System.EventHandler(this.btnReset_Click);
			// 
			// btnRecenter
			// 
			this.btnRecenter.Location = new System.Drawing.Point(298, 142);
			this.btnRecenter.Name = "btnRecenter";
			this.btnRecenter.Size = new System.Drawing.Size(94, 39);
			this.btnRecenter.TabIndex = 17;
			this.btnRecenter.Text = "Recenter area to camera";
			this.btnRecenter.UseVisualStyleBackColor = true;
			this.btnRecenter.Click += new System.EventHandler(this.btnRecenter_Click);
			// 
			// label6
			// 
			this.label6.AutoSize = true;
			this.label6.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.label6.Location = new System.Drawing.Point(4, 86);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(64, 13);
			this.label6.TabIndex = 20;
			this.label6.Text = "DEM size:";
			// 
			// label7
			// 
			this.label7.AutoSize = true;
			this.label7.Location = new System.Drawing.Point(6, 45);
			this.label7.Name = "label7";
			this.label7.Size = new System.Drawing.Size(30, 13);
			this.label7.TabIndex = 21;
			this.label7.Text = "Size:";
			// 
			// tbFilename
			// 
			this.tbFilename.Location = new System.Drawing.Point(75, 202);
			this.tbFilename.Name = "tbFilename";
			this.tbFilename.Size = new System.Drawing.Size(180, 20);
			this.tbFilename.TabIndex = 25;
			this.tbFilename.Text = "WW_Terrain";
			// 
			// tbDescription
			// 
			this.tbDescription.Location = new System.Drawing.Point(75, 229);
			this.tbDescription.Name = "tbDescription";
			this.tbDescription.Size = new System.Drawing.Size(309, 20);
			this.tbDescription.TabIndex = 26;
			this.tbDescription.Text = "World Wind Terrain";
			// 
			// label9
			// 
			this.label9.AutoSize = true;
			this.label9.Location = new System.Drawing.Point(18, 205);
			this.label9.Name = "label9";
			this.label9.Size = new System.Drawing.Size(52, 13);
			this.label9.TabIndex = 27;
			this.label9.Text = "Filename:";
			// 
			// label10
			// 
			this.label10.AutoSize = true;
			this.label10.Location = new System.Drawing.Point(7, 232);
			this.label10.Name = "label10";
			this.label10.Size = new System.Drawing.Size(63, 13);
			this.label10.TabIndex = 28;
			this.label10.Text = "Description:";
			// 
			// label11
			// 
			this.label11.AutoSize = true;
			this.label11.Location = new System.Drawing.Point(261, 205);
			this.label11.Name = "label11";
			this.label11.Size = new System.Drawing.Size(123, 13);
			this.label11.TabIndex = 29;
			this.label11.Text = "(extension will be added)";
			// 
			// label12
			// 
			this.label12.AutoSize = true;
			this.label12.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Underline, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.label12.Location = new System.Drawing.Point(108, 253);
			this.label12.Name = "label12";
			this.label12.Size = new System.Drawing.Size(160, 13);
			this.label12.TabIndex = 30;
			this.label12.Text = "Warning: overwrites existing files";
			// 
			// pbarExportProgress
			// 
			this.pbarExportProgress.Location = new System.Drawing.Point(2, 303);
			this.pbarExportProgress.Name = "pbarExportProgress";
			this.pbarExportProgress.Size = new System.Drawing.Size(392, 10);
			this.pbarExportProgress.Step = 1;
			this.pbarExportProgress.Style = System.Windows.Forms.ProgressBarStyle.Continuous;
			this.pbarExportProgress.TabIndex = 31;
			// 
			// comboDEMSize
			// 
			this.comboDEMSize.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboDEMSize.FormattingEnabled = true;
			this.comboDEMSize.Items.AddRange(new object[] {
            "150x150",
            "257x257",
            "513x513"});
			this.comboDEMSize.Location = new System.Drawing.Point(65, 83);
			this.comboDEMSize.Name = "comboDEMSize";
			this.comboDEMSize.Size = new System.Drawing.Size(63, 21);
			this.comboDEMSize.TabIndex = 32;
			this.comboDEMSize.SelectedIndexChanged += new System.EventHandler(this.comboDEMSize_SelectedIndexChanged);
			// 
			// cbTexture
			// 
			this.cbTexture.AutoSize = true;
			this.cbTexture.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.cbTexture.Location = new System.Drawing.Point(7, 112);
			this.cbTexture.Name = "cbTexture";
			this.cbTexture.Size = new System.Drawing.Size(69, 17);
			this.cbTexture.TabIndex = 33;
			this.cbTexture.Text = "Texture";
			this.cbTexture.UseVisualStyleBackColor = true;
			this.cbTexture.CheckedChanged += new System.EventHandler(this.cbTexture_CheckedChanged);
			// 
			// comboTextureSize
			// 
			this.comboTextureSize.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboTextureSize.FormattingEnabled = true;
			this.comboTextureSize.Items.AddRange(new object[] {
            "512x512",
            "1024x1024",
            "2048x2048"});
			this.comboTextureSize.Location = new System.Drawing.Point(35, 42);
			this.comboTextureSize.Name = "comboTextureSize";
			this.comboTextureSize.Size = new System.Drawing.Size(70, 21);
			this.comboTextureSize.TabIndex = 34;
			this.comboTextureSize.SelectedIndexChanged += new System.EventHandler(this.comboTextureSize_SelectedIndexChanged);
			// 
			// comboLayer
			// 
			this.comboLayer.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboLayer.FormattingEnabled = true;
			this.comboLayer.Location = new System.Drawing.Point(35, 12);
			this.comboLayer.MaxDropDownItems = 100;
			this.comboLayer.Name = "comboLayer";
			this.comboLayer.Size = new System.Drawing.Size(147, 21);
			this.comboLayer.TabIndex = 36;
			this.comboLayer.SelectedIndexChanged += new System.EventHandler(this.comboLayer_SelectedIndexChanged);
			// 
			// label8
			// 
			this.label8.AutoSize = true;
			this.label8.Location = new System.Drawing.Point(2, 16);
			this.label8.Name = "label8";
			this.label8.Size = new System.Drawing.Size(36, 13);
			this.label8.TabIndex = 35;
			this.label8.Text = "Layer:";
			// 
			// gbTexture
			// 
			this.gbTexture.Controls.Add(this.btnRefreshList);
			this.gbTexture.Controls.Add(this.comboLayer);
			this.gbTexture.Controls.Add(this.label8);
			this.gbTexture.Controls.Add(this.comboTextureSize);
			this.gbTexture.Controls.Add(this.label7);
			this.gbTexture.Enabled = false;
			this.gbTexture.Location = new System.Drawing.Point(2, 126);
			this.gbTexture.Name = "gbTexture";
			this.gbTexture.Size = new System.Drawing.Size(188, 70);
			this.gbTexture.TabIndex = 37;
			this.gbTexture.TabStop = false;
			// 
			// btnRefreshList
			// 
			this.btnRefreshList.Location = new System.Drawing.Point(111, 42);
			this.btnRefreshList.Name = "btnRefreshList";
			this.btnRefreshList.Size = new System.Drawing.Size(71, 23);
			this.btnRefreshList.TabIndex = 37;
			this.btnRefreshList.Text = "Refresh list";
			this.btnRefreshList.UseVisualStyleBackColor = true;
			this.btnRefreshList.Click += new System.EventHandler(this.btnRefreshList_Click);
			// 
			// TerrainExtractorForm
			// 
			this.AcceptButton = this.btnOK;
			this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
			this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
			this.CancelButton = this.btnCancel;
			this.ClientSize = new System.Drawing.Size(396, 315);
			this.Controls.Add(this.gbTexture);
			this.Controls.Add(this.cbTexture);
			this.Controls.Add(this.comboDEMSize);
			this.Controls.Add(this.pbarExportProgress);
			this.Controls.Add(this.label12);
			this.Controls.Add(this.label11);
			this.Controls.Add(this.label10);
			this.Controls.Add(this.label9);
			this.Controls.Add(this.tbDescription);
			this.Controls.Add(this.tbFilename);
			this.Controls.Add(this.label6);
			this.Controls.Add(this.btnRecenter);
			this.Controls.Add(this.btnReset);
			this.Controls.Add(this.numSize);
			this.Controls.Add(this.tbSize);
			this.Controls.Add(this.label5);
			this.Controls.Add(this.numRotation);
			this.Controls.Add(this.tbRotation);
			this.Controls.Add(this.label4);
			this.Controls.Add(this.numCenterLon);
			this.Controls.Add(this.numCenterLat);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.btnCancel);
			this.Controls.Add(this.btnOK);
			this.MaximizeBox = false;
			this.Name = "TerrainExtractorForm";
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
			this.Text = "Select Area To Cut Out...";
			((System.ComponentModel.ISupportInitialize)(this.numCenterLat)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numCenterLon)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.tbRotation)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numRotation)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numSize)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.tbSize)).EndInit();
			this.gbTexture.ResumeLayout(false);
			this.gbTexture.PerformLayout();
			this.ResumeLayout(false);
			this.PerformLayout();

		}

		#endregion


		public TerrainExtractorForm(TerrainExtractorPlugin plugin)
		{
			InitializeComponent();
			this.plugin = plugin;

			this.ResetBox();
			this.comboDEMSize.SelectedItem = this.comboDEMSize.Items[0];
			this.comboTextureSize.SelectedItem = this.comboTextureSize.Items[0];
			
			this.RefreshLayers(plugin.ParentApplication.WorldWindow.CurrentWorld.RenderableObjects);
			this.comboLayer.SelectedItem = (this.comboLayer.Items.Count > 1) ? this.comboLayer.Items[1] : this.comboLayer.Items[0];
			plugin.ImageLayerName = (string)this.comboLayer.SelectedItem;
		}

		/// <summary>
		/// Sets progress bar value (0-100)
		/// </summary>
		public int ProgBarValue
		{
			set { this.pbarExportProgress.Value = value; }
		}

		private void UpdateBox()
		{
			plugin.UpdateBox(numCenterLat.Value, numCenterLon.Value, numSize.Value, numRotation.Value);
		}

		private void ResetBox()
		{
			decimal centerLat;
			decimal centerLon;
			decimal size;
			plugin.ResetArea(out centerLat, out centerLon, out size);
			numCenterLat.Value = centerLat;
			numCenterLon.Value = centerLon;

			numSize.Value = size;
			numRotation.Value = 0;
			this.UpdateBox();
		}

		private void numCenterLat_ValueChanged(object sender, EventArgs e)
		{
			this.UpdateBox();
		}

		private void numCenterLon_ValueChanged(object sender, EventArgs e)
		{
			this.UpdateBox();
		}

		private void tbSize_ValueChanged(object sender, EventArgs e)
		{
			numSize.Value = (decimal)(Math.Pow(10, ((double)tbSize.Value / 1000)));
			this.UpdateBox();
		}

		private void tbRotation_ValueChanged(object sender, EventArgs e)
		{
			numRotation.Value = -1 * (decimal)tbRotation.Value;
			this.UpdateBox();
		}

		private void numSize_ValueChanged(object sender, EventArgs e)
		{
			tbSize.Value = (int)(Math.Log10((double)numSize.Value) * 1000);
			this.UpdateBox();
		}

		private void numRotation_ValueChanged(object sender, EventArgs e)
		{
			tbRotation.Value = -1 * (int)numRotation.Value;
			this.UpdateBox();
		}

		private void btnReset_Click(object sender, EventArgs e)
		{
			this.ResetBox();
		}

		private void btnOK_Click(object sender, EventArgs e)
		{
			if (comboTextureSize.SelectedIndex != 0)
			{
				if (MessageBox.Show("The selected texture size may take several minutes to export.  Continue?", "Export Size Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
				{
					plugin.WriteData(tbFilename.Text, tbDescription.Text);
				}
			}
			else
				plugin.WriteData(tbFilename.Text, tbDescription.Text);
		}

		private void updownDemSize_SelectedItemChanged(object sender, EventArgs e)
		{



		}

		private void btnRecenter_Click(object sender, EventArgs e)
		{
			decimal centerLat;
			decimal centerLon;
			plugin.Recenter(out centerLat, out centerLon);
			numCenterLat.Value = centerLat;
			numCenterLon.Value = centerLon;
			this.UpdateBox();
		}

		private void btnCancel_Click(object sender, EventArgs e)
		{
			this.Close();
		}

        	protected override void OnClosing(CancelEventArgs e)
        	{
			plugin.FormClosed();
            		base.OnClosing(e);
        	}



		private void comboDEMSize_SelectedIndexChanged(object sender, EventArgs e)
		{
			switch ((string)comboDEMSize.SelectedItem)
			{
				case "150x150":
					plugin.NumTerrainSamples = 150;
					break;

				case "257x257":
					plugin.NumTerrainSamples = 257;
					break;

				case "513x513":
					plugin.NumTerrainSamples = 513;
					break;
			}
		}

		private void cbTexture_CheckedChanged(object sender, EventArgs e)
		{
			this.gbTexture.Enabled = cbTexture.Checked;
			plugin.ExportTexture = cbTexture.Checked;
		}

		private void comboTextureSize_SelectedIndexChanged(object sender, EventArgs e)
		{
			switch ((string)comboTextureSize.SelectedItem)
			{
				case "512x512":
					plugin.NumImageSamples = 512;
					break;

				case "1024x1024":
					plugin.NumImageSamples = 1024;
					break;

				case "2048x2048":
					plugin.NumImageSamples = 2048;
					break;
			}

		}

		private void RefreshLayers(RenderableObjectList theList)
		{
			foreach (RenderableObject ro in theList.ChildObjects)
			{
				//If they're off, they can't see them to want them
				//Saves a lot of combo box clutter
				if (ro.IsOn)
				{
					//MessageBox.Show(ro.GetType().ToString());
					if (ro is QuadTileSet)
					{
						QuadTileSet curLayer = (QuadTileSet)ro;

						this.comboLayer.Items.Add(ro.Name);
						break;
					}
					if (ro is RenderableObjectList)
					{
						RefreshLayers((RenderableObjectList)ro);
					}
				}
			}
		}

		private void btnRefreshList_Click(object sender, EventArgs e)
		{
			this.comboLayer.Items.Clear();
			this.RefreshLayers(plugin.ParentApplication.WorldWindow.CurrentWorld.RenderableObjects);
			this.comboLayer.SelectedItem = (this.comboLayer.Items.Count > 1) ? this.comboLayer.Items[1] : this.comboLayer.Items[0];
			plugin.ImageLayerName = (string)this.comboLayer.SelectedItem;
		}

		private void comboLayer_SelectedIndexChanged(object sender, EventArgs e)
		{
			plugin.ImageLayerName = (string)comboLayer.SelectedItem;
		}
	}

	/// <summary>
	/// The plugin (main class)
	/// </summary>
	public class TerrainExtractorPlugin : WorldWind.PluginEngine.Plugin
	{
		/// <summary>
		/// Name displayed in layer manager
		/// </summary>
		public static string LayerName = "Select area";
		public int NumTerrainSamples = 150;
		public int NumImageSamples = 512;
		public string ImageLayerName = "I-Cubed ESAT World Landsat7 Mosaic";
		public bool ExportTexture = false;


		private LineFeature outline;
		private MenuItem menuItem;
		private TerrainExtractorForm teForm;
		private WorldWindow worldWindow;
		private WorldWind.Camera.CameraBase camera;
		private DrawArgs drawArgs;
		private double centerLat, centerLon, size, rotation;

		private string exeFileName = "TerrainViewer.exe";
		private string bilPath;
		private string imgPath;
		private string xmlPath;
		private string exePath;


		private Hashtable imageCache = new Hashtable();
		private bool debug = false;

		/// <summary>
		/// Plugin entry point - All plugins must implement this function
		/// </summary>
		public override void Load()
		{
			worldWindow = ParentApplication.WorldWindow;
			camera = worldWindow.DrawArgs.WorldCamera;
			drawArgs = worldWindow.DrawArgs;

			menuItem = new MenuItem("3D Cross Section...");
			menuItem.Click += new EventHandler(menuItemClicked);
			ParentApplication.PluginsMenu.MenuItems.Add(menuItem);

			exePath = Path.Combine(PluginDirectory, exeFileName);
		}

		/// <summary>
		/// Unloads our plugin
		/// </summary>
		public override void Unload()
		{
			ParentApplication.PluginsMenu.MenuItems.Remove(menuItem);
			if (outline != null)
				ParentApplication.WorldWindow.CurrentWorld.RenderableObjects.Remove(outline);
		}

		void menuItemClicked(object sender, EventArgs e)
		{
			teForm = new TerrainExtractorForm(this);
			teForm.Show();
		}

		public void UpdateBox(decimal cenLat, decimal cenLon, decimal s, decimal r)
		{
			centerLat = (double)cenLat;
			centerLon = (double)cenLon;
			size = (double)s;
			rotation = (double)r;

			// offset between center and corners of square
			double x = size * 1.4142 * Math.Cos((double)(45 - rotation) * 3.14159 / 180.0) / 2;
			double y = size * 1.4142 * Math.Sin((double)(45 - rotation) * 3.14159 / 180.0) / 2;

			Point3d nw = new Point3d(centerLon - x, centerLat + y, 0);
			Point3d sw = new Point3d(centerLon - y, centerLat - x, 0);
			Point3d se = new Point3d(centerLon + x, centerLat - y, 0);
			Point3d ne = new Point3d(centerLon + y, centerLat + x, 0);

			Point3d[] points = new Point3d[5];

			points[0] = nw;
			points[1] = sw;
			points[2] = se;
			points[3] = ne;
			points[4] = nw;

			if (outline == null)
			{
				outline = new LineFeature("3D Cross Section selection", worldWindow.CurrentWorld, points, Color.Red);
				outline.AltitudeMode = AltitudeMode.RelativeToGround;
				worldWindow.CurrentWorld.RenderableObjects.Add(outline);
			}
			else
			{
				outline.Points = points;
				outline.IsOn = true;
			}


		}

		public void WriteData(string filename, string description)
		{
			string bilFileName = @"maps\" + filename + "_SRTM.bil";
			string imgFileName = @"maps\" + filename + "_Texture.jpg";
			string xmlFileName = filename + ".xml";

			bilPath = Path.Combine(PluginDirectory, bilFileName);
			imgPath = Path.Combine(PluginDirectory, imgFileName);
			xmlPath = Path.Combine(PluginDirectory, xmlFileName);


			try
			{
				// *** Export elevation data to .bil file
				FileStream fs = new FileStream(bilPath, FileMode.Create);
				BinaryWriter bw = new BinaryWriter(fs);

				// lat and lon at start of first row
				double endLon = centerLon - size * 1.4142 * Math.Cos((double)(45 - rotation) * 3.14159 / 180.0) / 2;
				double endLat = centerLat + size * 1.4142 * Math.Sin((double)(45 - rotation) * 3.14159 / 180.0) / 2;

				// sample spacing in the direction of the row
				double stepSize = size / (NumTerrainSamples - 1);
				// samples per degree value to give to GetTerrainAt()
				double sampsPerDegree = NumTerrainSamples / size;

				// some trig shortcuts for later
				double ssin = stepSize * Math.Sin(rotation * 3.14159 / 180.0);
				double scos = stepSize * Math.Cos(rotation * 3.14159 / 180.0);

				int totalSamps = NumTerrainSamples * NumTerrainSamples;
				if (ExportTexture)
					totalSamps += NumImageSamples * NumImageSamples;

				int count = 0;
				double progress = 0;
				teForm.ProgBarValue = 0;

				for (int y = 0; y < NumTerrainSamples; y++)
				{
					for (int x = 0; x < NumTerrainSamples; x++)
					{
						// current lat and lon in this row
						double lat = endLat + ssin * x;
						double lon = endLon + scos * x;

						if (lon < -180)
							lon += 360;
						if (lon > 180)
							lon -= 360;

						short elev = (short)worldWindow.CurrentWorld.TerrainAccessor.GetElevationAt(lat, lon, sampsPerDegree);
						bw.Write(elev);
						count++;

						progress = count / totalSamps * 95;
						teForm.ProgBarValue = (int)progress;
					}

					// lat and lon at end of next row, column offsets are 90 deg. different from row offsets
					endLat -= scos;
					endLon += ssin;
				}

				bw.Close();
				fs.Close();

				// *** Export image

				if (ExportTexture)
				{
					// Find the layer we need
					QuadTileSet layer = (QuadTileSet)FindLayer(ParentApplication.WorldWindow.CurrentWorld.RenderableObjects,
								ImageLayerName);
					// Find the level we need
					int level = 0;
					// Find out tile size in pixels
					Color layerPixel = GetColorAt(layer, centerLat, centerLon, level); // Force one tile in imageCache
					double tileSizePixel = 512;
					if (imageCache.Count > 0)
					{
						foreach (Bitmap img in imageCache.Values) tileSizePixel = img.Width;
					}
					// Target samples per degree
					sampsPerDegree = NumImageSamples / size;
					// Layer samples per degre at level 0
					double layerSPD = tileSizePixel / layer.ImageStores[0].LevelZeroTileSizeDegrees;
					while (level < layer.ImageStores[0].LevelCount - 1 && layerSPD < sampsPerDegree)
					{
						layerSPD *= 2;
						level += 1;
					}
					if (level > layer.ImageStores[0].LevelCount) level = layer.ImageStores[0].LevelCount;
					//MessageBox.Show("Layer :  " + layer.Name + " - Level : " + level, "Layer list", MessageBoxButtons.OK, MessageBoxIcon.Error);

					// lat and lon at start of first row
					endLon = centerLon - size * 1.4142 * Math.Cos((double)(45 - rotation) * 3.14159 / 180.0) / 2;
					endLat = centerLat + size * 1.4142 * Math.Sin((double)(45 - rotation) * 3.14159 / 180.0) / 2;

					// sample spacing in the direction of the row
					stepSize = size / (NumImageSamples - 1);
					// some trig shortcuts for later
					ssin = stepSize * Math.Sin(rotation * 3.14159 / 180.0);
					scos = stepSize * Math.Cos(rotation * 3.14159 / 180.0);

					// New bitmap
					Bitmap image = new Bitmap(NumImageSamples, NumImageSamples, System.Drawing.Imaging.PixelFormat.Format32bppArgb);

					debug = true;

					for (int y = 0; y < NumImageSamples; y++)
					{
						for (int x = 0; x < NumImageSamples; x++)
						{
							// current lat and lon in this row
							double lat = endLat + ssin * x;
							double lon = endLon + scos * x;

							if (lon < -180)
								lon += 360;
							if (lon > 180)
								lon -= 360;

							Color color = GetColorAt(layer, lat, lon, level);
							debug = false;
							image.SetPixel(x, y, color);
							count++;

							progress = count / totalSamps * 95;
							teForm.ProgBarValue = (int)progress;
						}

						// lat and lon at start of next row, column offsets are 90 deg. different from row offsets
						endLat -= scos;
						endLon += ssin;
					}
					// Save image
					image.Save(imgPath, System.Drawing.Imaging.ImageFormat.Jpeg);
					// Images cleanup
					image.Dispose();
					DisposeImageCache();

				}

				// *** Write xml meta data
				XmlDocument doc = new XmlDocument();
				doc.CreateXmlDeclaration("1.0", "ISO-8859-1", null);

				XmlElement root = doc.CreateElement("maps");
				doc.AppendChild(root);

				XmlElement map = doc.CreateElement("map");
				map.SetAttribute("name", description);
				root.AppendChild(map);

				XmlElement terrain = doc.CreateElement("terrain");
				terrain.SetAttribute("demfile", bilFileName);

				if (ExportTexture)
					terrain.SetAttribute("texturefile", imgFileName);
				else
					terrain.SetAttribute("texturefile", @"colors\Geo_Water_1.png");

				terrain.SetAttribute("altscale", "1.0");
				terrain.SetAttribute("centerlatitude", centerLat.ToString(CultureInfo.InvariantCulture));
				terrain.SetAttribute("centerlongitude", centerLon.ToString(CultureInfo.InvariantCulture));
				terrain.SetAttribute("span", size.ToString(CultureInfo.InvariantCulture));
				terrain.SetAttribute("heading", rotation.ToString(CultureInfo.InvariantCulture));
				terrain.SetAttribute("worldradius", worldWindow.CurrentWorld.EquatorialRadius.ToString(CultureInfo.InvariantCulture));
				map.AppendChild(terrain);

				doc.Save(xmlPath);

				// Start Viewer here
				ProcessStartInfo ps = new ProcessStartInfo(exePath, xmlFileName);
				ps.WorkingDirectory = PluginDirectory;
				//ps.UseShellExecute = false;
				Process.Start(ps);

				// Show some progress
				progress = 100;
				teForm.ProgBarValue = (int)progress;
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "Export error");
			}
		}

		// Get pixel color at some lat/lon from a layer's cache
		public Color GetColorAt(QuadTileSet layer, double lat, double lon, int level)
		{
			Color color = Color.White;
			// Find out tile size at that level
			double tileSizeDegree = layer.ImageStores[0].LevelZeroTileSizeDegrees;
			if (level > 0) for (int i = 0; i < level; i++) tileSizeDegree /= 2;
			// Tile row/col
			//int row = MathEngine.GetRowFromLatitude(lat, tileSizeDegree); // Broken code !!!
			//int col = MathEngine.GetColFromLongitude(lon, tileSizeDegree);
			int row = (int)Math.Floor((lat + 90) / tileSizeDegree);
			int col = (int)Math.Floor((lon + 180) / tileSizeDegree);
			// Tile boundaries
			double south = -90.0f + row * tileSizeDegree;
			double north = south + tileSizeDegree;
			double west = -180.0f + col * tileSizeDegree;
			double east = west + tileSizeDegree;
			// Look in cache first
			string key = level.ToString() + row.ToString() + col.ToString();
			Bitmap img = (Bitmap)imageCache[key];
			if (img == null)
			{
				// Use a QuadTile to get to the tile image
				QuadTile qt = new QuadTile(south, north, west, east, level, layer);
				string tilePath = layer.ImageStores[0].GetLocalPath(qt);
				if (File.Exists(tilePath))
				{
					Texture texture = layer.ImageStores[0].LoadFile(qt);
					GraphicsStream gs = TextureLoader.SaveToStream(ImageFileFormat.Bmp, texture);
					img = new Bitmap(gs);
					imageCache.Add(key, img);
					texture.Dispose();
				}
				qt.Dispose();
			}
			// Do we have an image to get pixels from ?
			if (img != null)
			{
				// Decimal pixel position
				double x = (lon - west) / tileSizeDegree * img.Width;
				double y = (north - lat) / tileSizeDegree * img.Height;
				// Clamp to bitmap - just in case..
				if (x >= img.Width) x = img.Width - 1;
				if (y >= img.Height) y = img.Height - 1;
				if (x < 0) x = 0;
				if (y < 0) y = 0;
				// Average four corners / closest pixels
				int xNW = (int)Math.Floor(x);
				int yNW = (int)Math.Floor(y);
				float xFactor = (float)(x - xNW);
				float yFactor = (float)(y - yNW);
				Color NW = img.GetPixel(xNW, yNW);
				Color NE = (xNW + 1) < img.Width ? img.GetPixel(xNW + 1, yNW) : NW;
				Color SW = (yNW + 1) < img.Height ? img.GetPixel(xNW, yNW + 1) : NW;
				Color SE = (xNW + 1) < img.Width && (yNW + 1) < img.Height ?
					img.GetPixel(xNW + 1, yNW + 1) : NW;
				Color N = Color.FromArgb(
					(byte)((float)NW.R * (1f - xFactor) + (float)NE.R * xFactor),
					(byte)((float)NW.G * (1f - xFactor) + (float)NE.G * xFactor),
					(byte)((float)NW.B * (1f - xFactor) + (float)NE.B * xFactor));
				Color S = Color.FromArgb(
					(byte)((float)SW.R * (1f - xFactor) + (float)SE.R * xFactor),
					(byte)((float)SW.G * (1f - xFactor) + (float)SE.G * xFactor),
					(byte)((float)SW.B * (1f - xFactor) + (float)SE.B * xFactor));
				color = Color.FromArgb(
					(byte)((float)N.R * (1f - yFactor) + (float)S.R * yFactor),
					(byte)((float)N.G * (1f - yFactor) + (float)S.G * yFactor),
					(byte)((float)N.B * (1f - yFactor) + (float)S.B * yFactor));
			}
			return color;
		}

		public void DisposeImageCache()
		{
			if (imageCache.Count > 0)
			{
				foreach (Bitmap img in imageCache.Values) img.Dispose();
				imageCache.Clear();
			}

		}

		/// <summary>
		/// Find a layer ref with part of its name (layer an parents must be On)
		/// </summary>
		public RenderableObject FindLayer(RenderableObject ro, string name)
		{
			string s = "";
			if(!ro.IsOn) return null;
			if (ro is WorldWind.Renderable.RenderableObjectList)
			{
				RenderableObjectList rol = (RenderableObjectList)ro;
				for (int i = 0; i < rol.ChildObjects.Count; i++)
				{
					RenderableObject l = (RenderableObject)rol.ChildObjects[i];
					if (l is RenderableObjectList)
					{
						RenderableObject found = FindLayer(l, name);
						if (found != null) return found;
					}
					else
					{
						s += l.Name + ", ";
						if (l.Name.IndexOf(name) != -1 && l.IsOn) return l;
					}
				}
			}
			else
			{
				if (ro.Name.IndexOf(name) != -1 && ro.IsOn) return ro;
			}
			//MessageBox.Show("Names :  " + s, "Layer list", MessageBoxButtons.OK, MessageBoxIcon.Error );
			return null;
		}


		public void ResetArea(out decimal cenLat, out decimal cenLon, out decimal s)
		{
			centerLat = camera.Latitude.Degrees;
			cenLat = (decimal)centerLat;
			centerLon = camera.Longitude.Degrees;
			cenLon = (decimal)centerLon;
			size = 1.8 * 180.0 / 3.14159 * Math.Atan(camera.Altitude / worldWindow.CurrentWorld.EquatorialRadius * 0.4142);
			if (size > 60)
				size = 60;
			if (size < 0.01)
				size = 0.01;

			s = (decimal)size;
		}

		public void Recenter(out decimal cenLat, out decimal cenLon)
		{
			centerLat = camera.Latitude.Degrees;
			cenLat = (decimal)centerLat;
			centerLon = camera.Longitude.Degrees;
			cenLon = (decimal)centerLon;
		}

		public void ClearSelectionBox()
		{
			outline.Dispose();
			outline = null;
		}

		public void FormClosed()
		{
			if(outline != null) {
				outline.Points = new Point3d[0];
				outline.IsOn = false;
			}
		}
	}
}
